#!/usr/bin/env bash

# This is the master of model one training
# Parameters needed:
echo "--------------------------------------------------------"
echo "Usage of masters"
echo "modelOneMaster <template> <watchdir> <collectdir> <logdir> "
echo "     <output_t> <output_a> <output_d> <output_n> <output_z>"
echo "     <runs> <tmp> <childlist> <seed_t> <seed_a>"
echo "     [seed_d] [seed_n] [seed_pz]"
echo "1 <template> The template file for the normalization"
echo "2 <watchdir> The directory to watch the output of childs"
echo "3 <collectdir> The directory to collect output of childs"
echo "4 <logdir> The directory to collect log of childs"
echo "5 <output_t> Final T Table for output"
echo "6 <output_a> Final A Table for output"
echo "7 <output_d> Final D Table for output"
echo "8 <output_n> Final N Table for output"
echo "9 <output_z> Final p0 Table for output"
echo "10 <runs> The running string, 0123 for the model of each run"
echo "          for example, 01112333 will run three IBM model 3 training"
echo "          and 3 IBM model 4 training"
echo "11 <tmp> the temporary dir (Child process should be able to access)"
echo "12 <childlist> is the list of childs"
echo "13 <seed_t> is the previous t model"
echo "14 <seed_a> is the previous a model"
echo "15 <seed_hmm> is the previous hmm model"
echo "16 [seed_d] is the previous d model"
echo "17 [seed_n] is the previous n model"
echo "18 [seed_pz] is the previous p0 model"
echo "--------------------------------------------------------"
echo 

if [ -e $PWD/defines.inc ];
then
        source $PWD/defines.inc
else
	MODEL_ONE_NORM=/SMT/giza/view_source/newgiza/src/para/renomalizeModel1
	ATABLE_EXEC=/SMT/giza/view_source/newgiza/src/para/renomalizeAModel
	NDPTABLE_EXEC=/SMT/giza/view_source/newgiza/src/para/renomalizeNDP
	CHMM_EXEC=/SMT/giza/view_source/newgiza/src/para/combine_hmm
fi

if [ $# -lt "14" ];
then
	echo " NO! I need your command!"
	exit 1
fi

TEMPL=$1
WATCHDIR=$2
COLLDIR=$3
LOGDIR=$4
OUTPUT=$5
OUTPUT_A=$6
OUTPUT_D=$7
OUTPUT_N=$8
OUTPUT_P=$9
RUNS=${10}
TMPDIR=${11}
CHILDLIST=${12}
SEEDMODEL=${13}
SEEDMODEL_A=${14}
SEEDMODEL_H=${15}
SEEDMODEL_D=${16}
SEEDMODEL_N=${17}
SEEDMODEL_P=${18}


LOGFILE=${LOGDIR}/model34_master.log.$$

echo "If I die, check " $LOGFILE


echo "... Checking template" ${TEMPL}
if [ -r ${TEMPL} ]; # is template exists?
then
	echo " OK"
else
	echo " NO! The template cannot be read! "
	exit 1
fi

echo "... Checking watching dir" ${WATCHDIR}
if [ -d ${WATCHDIR} ]; # is watch dir exists
then
	echo " OK cleaning"
	rm -f ${WATCHDIR}/*
else
	echo " NO! The watching dir cannot be accessed"
	exit 1
fi

echo "... Checking collecting dir" ${COLLDIR}
if [ -d ${COLLDIR} ]; # is collection OK?
then
	echo " OK"
else
	echo " NO! The collecting dir cannot be accessed"
	exit 1
fi

echo "... Checking log dir" ${LOGDIR}
if [ -d ${LOGDIR} ]; # is log OK?
then
	echo " OK"
else
	echo " NO! The log dir cannot be accessed"
	exit 1
fi

echo "... Checking temporary dir" ${TMPDIR}
if [ -d ${TMPDIR} ]; # is tmp OK?
then
	echo " OK"
else
	echo " NO! The temporary dir cannot be accessed"
	exit 1
fi

echo "... Checking child list" ${CHILDLIST}
if [ -r ${CHILDLIST} ]; # is childlist OK?
then
	echo " OK"
else
	echo " NO! The child list cannot be accessed"
	exit 1
fi

echo "... Checking seed t model" ${SEEDMODEL}
if [ -r ${SEEDMODEL} ]; # is t model OK
then
	echo " OK"
else
	echo " NO! The previous t model cannot be accessed"
	exit 1
fi

echo "... Checking seed a model" ${SEEDMODEL_A}
if [ -r ${SEEDMODEL_A} ]; # is t model OK
then
	echo " OK"
else
	echo " NO! The previous a model cannot be accessed"
	exit 1
fi



touch ${OUTPUT}

if [ -w ${OUTPUT} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT}
	echo " OK"
else
	echo " NO! I will not be able to write output file" ${OUTPUT}
	exit 1
fi

rm -f ${OUTPUT}

touch ${OUTPUT_A}

if [ -w ${OUTPUT_A} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_A}
	echo " OK"
else
	echo " NO! I will not be able to write output file" ${OUTPUT_A}
	exit 1
fi

rm -f ${OUTPUT_A}

touch ${OUTPUT_D}

if [ -w ${OUTPUT_D} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_D}
	echo " OK"
else
	echo " NO! I will not be able to write output file" ${OUTPUT_D}
	exit 1
fi
rm -f ${OUTPUT_D}

touch ${OUTPUT_N}

if [ -w ${OUTPUT_N} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_N}
	echo " OK"
else
	echo " NO! I will not be able to write output file" ${OUTPUT_N}
	exit 1
fi
rm -f ${OUTPUT_N}
touch ${OUTPUT_P}

if [ -w ${OUTPUT_P} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_P}
	echo " OK"
else
	echo " NO! I will not be able to write output file" ${OUTPUT_P}
	exit 1
fi
rm -f ${OUTPUT_P}



ROUNDS=${#RUNS}

echo "We will do " $ROUNDS " rounds of training"


# If we survives here, let's start running

CUR=0 # Current round
#Total number of childs

TOTAL_CHILD=`wc -l $CHILDLIST | awk '{print $1}' `

echo "Total number of children : " $TOTAL_CHILD

# Here we read configure of all the child
((TOTAL_CHILD = TOTAL_CHILD + 1))
CN=1
while [ $CN -lt $TOTAL_CHILD ];
do
	host[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $1}'`
	exe[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $2}'`
	conf[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $3}'`
	(( CN = CN + 1 ))
done

CRIT=$ROUNDS
(( CRIT = CRIT - 1 ))
while [ $CUR -lt $ROUNDS ];
do
	if [ $CUR == $CRIT ];
	then
		PRUNE=1
	else
		PRUNE=0
	fi
	
	MODE=${RUNS:${CUR}:1}
	
	if [ $MODE == "0" ];
	then 
		ALSUF=AH3
	fi
	if [ $MODE == "1" ];
	then
		ALSUF=A3
	fi		
	if [ $MODE == "2" ];
	then
		ALSUF=A34
	fi
	if [ $MODE == "3" ];
	then
		ALSUF=A4
	fi	
	echo "*Start running round " $CUR " with mode " $MODE
	if [ $CUR -gt "0" ]; # not first round,
	then
		(( PP = CUR - 1 ))
		PMODE=${RUNS:${PP}:1}
		prev_t=${TMPDIR}/H34_R_${PP}_M_${PMODE}_TTABLE.tbl
		prev_a=${TMPDIR}/H34_R_${PP}_M_${PMODE}_ATABLE.tbl
		prev_d=${TMPDIR}/H34_R_${PP}_M_${PMODE}_DTABLE.tbl
		prev_n=${TMPDIR}/H34_R_${PP}_M_${PMODE}_NTABLE.tbl
		prev_p=${TMPDIR}/H34_R_${PP}_M_${PMODE}_ZTABLE.tbl
		#prev_h=${TMPDIR}/H34_R_${PP}_M_${PMODE}_HTABLE.tbl
#well, it seems the h table is not gonna updated
		prev_h=$SEEDMODEL_H
	else
		prev_t=$SEEDMODEL
		prev_a=$SEEDMODEL_A
		prev_d=$SEEDMODEL_D
		prev_n=$SEEDMODEL_N
		prev_p=$SEEDMODEL_P
		prev_h=$SEEDMODEL_H
	fi
	#################################################
	# Start all the process
	#################################################
	rm -f ${WATCHDIR}/*
	CN=1
	while [ $CN -lt $TOTAL_CHILD ];
	do
		output[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.diff
		acount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.acount
		dcount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.dcount
		ncount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.ncount
		hcount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.hcount
		pcount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.pcount
		signal[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish
		align[${CN}]=${ALIGN_DIR}/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.${ALSUF}
		start[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.starting
		fail[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.failed
		log[${CN}]=$LOGDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.log
		echo "cd" $PWD > ${CN}.tmp
		echo ${exe[${CN}]} ${conf[${CN}]} ${output[${CN}]}  ${acount[${CN}]} ${dcount[${CN}]} ${ncount[${CN}]} ${pcount[${CN}]} ${hcount[${CN}]} ${MODE} ${signal[${CN}]} ${log[${CN}]} ${prev_t} ${prev_a} ${prev_h} ${prev_d} ${prev_n} ${prev_p} >> ${CN}.tmp
		command[${CN}]=`echo ${SUBMIT_ALIGN}  source $PWD/${CN}.tmp`
                           #           host         exe            configure     output_t          output_a         output_d        output_n          output_pz         output_h        mode     signal            log         prev_t    prev_a    prev_h   prev_d   prev_n    prev_p    
            touch  ${start[${CN}]}	
            echo ${command[${CN}]}
            ${command[${CN}]} >>${LOGFILE}&
            echo Child ${CN} Started on ${host[$CN]}
            (( CN = CN + 1 ))
	done
	sleep 1
	################################################
	# Watch what's going on
	################################################
	F=0
	while [ $F = "0" ]
	do
		sleep 10
		CN=1
		F=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			if [ -e ${start[${CN}]} ]; # Starting process failed
			then
				echo Starting child $CN is pending!
				echo  ${host[${CN}]}  ${exe[${CN}]}  ${conf[${CN}]}
				F=0
			fi	
			if [ -e ${signal[${CN}]} ]; # Finished
			then
				echo Child $CN on ${host[${CN}]} completed
			else
				if [ -e ${fail[${CN}]} ];
				then
					echo Child $CN on ${host[${CN}]} FAILED!
					echo Stop process and check ${log[${CN}]}
					F=-1
				else
					F=0
					echo Child $CN on ${host[${CN}]} Running!
				fi
			fi
			(( CN = CN + 1 ))
		done
	done
	
	if [ $F -lt "0" ]; # error occur!
	then
		echo "SAD..."
		exit 1
	else
		out_t=${TMPDIR}/H34_R_${CUR}_M_${MODE}_TTABLE.tbl
		out_a=${TMPDIR}/H34_R_${CUR}_M_${MODE}_ATABLE.tbl
		out_d=${TMPDIR}/H34_R_${CUR}_M_${MODE}_DTABLE.tbl
		out_n=${TMPDIR}/H34_R_${CUR}_M_${MODE}_NTABLE.tbl
		out_p=${TMPDIR}/H34_R_${CUR}_M_${MODE}_ZTABLE.tbl
		out_h=${TMPDIR}/H34_R_${CUR}_M_${MODE}_HTABLE.tbl

		# Do normalization
	    signal_t=$WATCHDIR/R_${CUR}_COMBINE_T_TABLE.finish
	    start_t=${signal_t}.starting
		fail_t=${signal_t}.failed
		log_t=$LOGDIR/R_${CUR}_COMBINE_T_TABLE.log

		
		CONF=${TMPDIR}/${CUR}_T_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm1 ${CUR}>> $CONF
		echo outputttable ${out_t}>>$CONF
		echo actmodel1 0 >> $CONF
		echo prn ${PRUNE} >> $CONF
#		else
#			echo actmodel1 0 >> $CONF # if we want actual table, then do it here
#		fi
		echo prevtmodel ${prev_t}>>$CONF
#		cml=`echo ${MODEL_ONE_NORM} ${CONF}`
		cml=`echo ${SUBMIT_TTABLE} ${MERGE_SCRIPT} ${MODEL_ONE_NORM} ${CONF} ${signal_t} ${log_t}`
		
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${output[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo "Starting normalization of T Model, invoking:"
		echo $cml
		# IS T MODEL OK?
		if ${cml}>>${LOGFILE};
		then	
#			echo "Normalization of T Model OK"
#			echo "Output written to " ${out_t}
			echo " Normalization of T Model Started"
		else
			echo "Normalization of T Model FAILED"
			echo "Exit!"
			exit 1;
		fi
		
# combine a table

	    signal_a=$WATCHDIR/R_${CUR}_COMBINE_A_TABLE.finish
	    start_a=${signal_a}.starting
		touch ${start_a}	    
		fail_a=${signal_a}.failed
		log_a=$LOGDIR/R_${CUR}_COMBINE_A_TABLE.log		
		
		CONF=${TMPDIR}/${CUR}_A_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm2 ${CUR}>> $CONF
		echo outputatable ${out_a}>>$CONF
		echo actmodel1 0 >> $CONF
		echo prevamodel ${prev_a}>>$CONF
		echo prn ${PRUNE} >> $CONF
		cml=`echo ${SUBMIT_NORM} ${MERGE_SCRIPT} ${ATABLE_EXEC} ${CONF} ${signal_a} ${log_a}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${acount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo "Starting normalization of A Model, invoking:"
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo " Normalization of A Model Started"
		else
			echo "Normalization of A Model FAILED"
			echo "Exit!"
			exit 1;
		fi

# NTable
	    signal_n=$WATCHDIR/R_${CUR}_COMBINE_N_TABLE.finish
	    start_n=${signal_n}.starting
		touch ${start_n}	    
		fail_n=${signal_n}.failed
		log_n=$LOGDIR/R_${CUR}_COMBINE_N_TABLE.log	

		CONF=${TMPDIR}/${CUR}_N_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo "opt 0"  >> $CONF
		echo outputntable ${out_n}>>$CONF
		echo actmodel1 0 >> $CONF
		echo prevnmodel ${prev_n}>>$CONF
		echo prn ${PRUNE} >> $CONF
#		cml=`echo ${NDPTABLE_EXEC} ${CONF}`
		cml=`echo ${SUBMIT_NORM} ${MERGE_SCRIPT} ${NDPTABLE_EXEC} ${CONF} ${signal_n} ${log_n}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${ncount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo "Starting normalization of N Model, invoking:"
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo " Normalization of N Model Started"
			#echo "Normalization of N Model OK"
			#echo "Output written to " ${out_n}
		else
			echo "Normalization of N Model FAILED"
			echo "Exit!"
			exit 1;
		fi
		
        if [ $MODE -gt 1 ];
		then
		
		    signal_d4=$WATCHDIR/R_${CUR}_COMBINE_D4_TABLE.finish
		    start_d4=${signal_d4}.starting
			touch ${start_d4}	    
			fail_d4=${signal_d4}.failed
			log_d4=$LOGDIR/R_${CUR}_COMBINE_D4_TABLE.log	

			CONF=${TMPDIR}/${CUR}_D4_NORMAL.gizacfg
			rm -f $CONF
			cat $TEMPL > $CONF
			echo "opt 3"  >> $CONF
			echo outputntable ${out_d}>>$CONF
			echo actmodel1 0 >> $CONF
			echo prevnmodel ${prev_d}>>$CONF
			echo prn ${PRUNE} >> $CONF
			#cml=`echo ${NDPTABLE_EXEC} ${CONF}`
			cml=`echo ${SUBMIT_NORM} ${MERGE_SCRIPT} ${NDPTABLE_EXEC} ${CONF} ${signal_d4} ${log_d4}`
			CN=1
			while [ $CN -lt $TOTAL_CHILD ];
			do
				cml=`echo ${cml}  ${dcount[${CN}]} | tr -s / /`
				(( CN = CN + 1 ))
			done
			echo "Starting normalization of D4 Model, invoking:"
			echo $cml
			if ${cml}>>${LOGFILE};
			then
				echo " Normalization of D4 Model Started"
			else
				echo "Normalization of D4 Model FAILED"
				echo "Exit!"
				exit 1;
			fi
		fi
		    
		signal_d=$WATCHDIR/R_${CUR}_COMBINE_D_TABLE.finish
		start_d=${signal_d}.starting
		touch ${start_d}	    
		fail_d=${signal_d}.failed
		log_d=$LOGDIR/R_${CUR}_COMBINE_D_TABLE.log	
			
		CONF=${TMPDIR}/${CUR}_D_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo "opt 1"  >> $CONF
		echo outputntable ${out_d}>>$CONF
		echo actmodel1 0 >> $CONF
		echo prevnmodel ${prev_d}>>$CONF
		echo prn ${PRUNE} >> $CONF
		cml=`echo ${SUBMIT_NORM} ${MERGE_SCRIPT} ${NDPTABLE_EXEC} ${CONF} ${signal_d} ${log_d}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${dcount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo "Starting normalization of D Model, invoking:"
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo " Normalization of D4 Model Started"
			#echo "Output written to " ${out_d}
		else
			echo "Normalization of D Model FAILED"
			echo "Exit!"
			exit 1;
		fi
		
# This one can run on any machine, don't bother to submit it!		
		CONF=${TMPDIR}/${CUR}_P_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo "opt 2"  >> $CONF
		echo outputntable ${out_p}>>$CONF
		echo actmodel1 0 >> $CONF
		echo prevnmodel ${prev_p}>>$CONF
		cml=`echo ${NDPTABLE_EXEC} ${CONF}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${pcount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo "Starting normalization of P0, invoking:"
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo "Normalization of P0 Model OK"
			echo "Output written to " ${out_p}
		else
			echo "Normalization of P0 Model FAILED"
			echo "Exit!"
			exit 1;
		fi
		
		F=0
		while [ $F = 0 ]
		do
			sleep 5
			F=1
			if [ -e ${start_t} ]; # Starting process failed
        		then
				echo Normalization of TTable is pending...
				F=0
			else
				if [ -e ${signal_t} ]; # Finished
				then
					echo Normalization of TTable is completed
				else
					if [ -e ${fail_t} ];
					then
						echo Normalization of TTable Failed, cannot continue!
						echo Exitting
						exit 1
					else
						echo  Normalization of TTable  is running.
						F=0
					fi
				fi
			fi
			
			if [ -e ${start_a} ]; # Starting process failed
        		then
				echo Normalization of ATable is pending...
				F=0
			else
		
				if [ -e ${signal_a} ]; # Finished
				then
					echo Normalization of ATable is completed
				else
					if [ -e ${fail_a} ];
					then
						echo Normalization of ATable Failed, cannot continue!
						echo Exitting
						exit 1
					else
						echo  Normalization of ATable  is running.
						F=0
					fi
				fi
			fi
			
			if [ -e ${start_n} ]; # Starting process failed
        		then
				echo Normalization of NTable is pending...
				F=0
			else
		
				if [ -e ${signal_n} ]; # Finished
				then
					echo Normalization of NTable is completed
				else
					if [ -e ${fail_n} ];
					then
						echo Normalization of NTable Failed, cannot continue!
						echo Exitting
						exit 1
					else
						echo  Normalization of NTable  is running.
						F=0
					fi
				fi
			fi
			
			if [ $MODE -gt 1 ];
			then
				if [ -e ${start_d4} ]; # Starting process failed
        			then
					echo Normalization of D4Table is pending...
					F=0
				else
					if [ -e ${signal_d4} ]; # Finished
					then
						echo Normalization of D4Table is completed
					else
						if [ -e ${fail_d4} ];
						then
							echo Normalization of D4Table Failed, cannot continue!
							echo Exitting
							exit 1
						else
							echo  Normalization of D4Table  is running.
							F=0
						fi
					fi
				fi
			fi
		done		
	fi			
	(( CUR = CUR + 1 ))	
done

if [ -e ${out_t} ];
then
	if [ -e ${out_a} ]
	then
		echo "No more iteration"
		echo "Copying last round's table to " $OUTPUT
		cp -fv ${out_a} ${OUTPUT_A}
		cp -fv ${out_t} ${OUTPUT}
		cp -fv ${out_t}.inv ${OUTPUT}.inv
		cp -fv ${out_n} ${OUTPUT_N}
		cp -fv ${out_d} ${OUTPUT_D}
		cp -fv ${out_p} ${OUTPUT_P}
		cp -fv ${out_d}.4 ${OUTPUT_D}.4
		cp -fv ${out_d}.4.b ${OUTPUT_D}.4.b
	fi
fi

# Combine Alignment

CN=1
cml="cat "
while [ $CN -lt $TOTAL_CHILD ];
do
	cml=`echo ${cml}  ${align[${CN}]} | tr -s / /`
      (( CN = CN + 1 ))
done

#cml=`echo ${cml} "> ${OUTPUT_ALIGN}"`

echo "Combining Alignment using :" ${cml} to ${OUTPUT_ALIGN}
${cml} > ${OUTPUT_ALIGN}
source ${CLEANING}
